@@ -6,8 +6,7 @@ from collections.abc import Callable
 from aiohttp import ClientResponse
 
 from .auth import AbstractAuth
-from .common import HomeConnectError
-
+from .common import ConditionalLogger, HomeConnectError, GlobalStatus
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -47,24 +46,52 @@ class HomeConnectApi():
     def __init__(self, auth:AbstractAuth, lang:str=None):
         self._auth = auth
         self._lang = lang
+        self._call_counter = 0
 
-
-    async def _async_request(self, method, endpoint, data=None) -> ApiResponse:
+    async def _async_request(self, method:str, endpoint:str, data=None) -> ApiResponse:
         """ Main function to call the Home Connect API over HTTPS """
+        method = method.upper()
         retry = 3
         response = None
         while retry:
             try:
+                self._call_counter += 1
+
+                if ConditionalLogger.ismode(ConditionalLogger.LogMode.REQUESTS):
+                    if data:
+                         _LOGGER.debug("\nHTTP %s %s (try=%d count=%d)\n%s\n", method, endpoint, 4-retry, self._call_counter, data)
+                    else:
+                        _LOGGER.debug("\nHTTP %s %s (try=%d count=%d)\n", method, endpoint, 4-retry, self._call_counter)
+
                 response = await self._auth.request(method, endpoint, self._lang,  data=data)
+
+
+                # if self._log_mode and (self._log_mode & LogMode.REQUESTS) and (self._log_mode & LogMode.RESPONSES):
+                #     _LOGGER.debug("\nHTTP RESPONSE [%d] (try=%d count=%d) ====>\n%s\n", response.status,4-retry, self._call_counter, await response.text(encoding="UTF-8"))
+                #     if data:
+                #         _LOGGER.debug("\nHTTP %s %s [%d] (try=%d count=%d)\n%s\nResponse ====>\n%s", method, endpoint, response.status, 4-retry, self._call_counter, data, await response.text(encoding="UTF-8"))
+                #     else:
+                #         _LOGGER.debug("\nHTTP %s %s [%d] (try=%d count=%d)\nResponse ====>\n%s", method, endpoint, response.status, 4-retry, self._call_counter, await response.text(encoding="UTF-8"))
+                # elif self._log_mode and (self._log_mode & LogMode.REQUESTS) and data:
+                #     _LOGGER.debug("\nHTTP %s %s [%d] (try=%d count=%d)\n%s", method, endpoint, response.status, 4-retry, self._call_counter, data)
+                if ConditionalLogger.ismode(ConditionalLogger.LogMode.RESPONSES):
+                    if response.content_length and response.content_length>0:
+                        _LOGGER.debug("\nHTTP %s %s (try=%d count=%d) [%d %s] ====>\n%s\n", method, endpoint, 4-retry, self._call_counter, response.status, response.reason, await response.text(encoding="UTF-8"))
+                    else:
+                        _LOGGER.debug("\nHTTP %s %s (try=%d count=%d) [%d %s]\n", method, endpoint, 4-retry, self._call_counter, response.status, response.reason)
+                else:
+                    _LOGGER.debug("HTTP %s %s (try=%d count=%d) [%d]", method, endpoint, 4-retry, self._call_counter, response.status)
                 if response.status == 429:    # Too Many Requests
                     wait_time = response.headers.get('Retry-After')
                     _LOGGER.debug('HTTP Error 429 - Too Many Requests. Sleeping for %s seconds and will retry', wait_time)
+                    GlobalStatus.set_status(GlobalStatus.Status.BLOCKED, int(wait_time))
                     await asyncio.sleep(int(wait_time)+1)
-                elif method in ["put", "delete"] and response.status == 204:
+                    GlobalStatus.unset_status(GlobalStatus.Status.BLOCKED)
+                elif method in ["PUT", "DELETE"] and response.status == 204:
                     result = self.ApiResponse(response, None)
                     return result
                 else:
-                    result = self.ApiResponse(response,  await response.json())
+                    result = self.ApiResponse(response,  await response.json(encoding='UTF-8'))
                     if result.status == 401 or result.status >= 500: # Unauthorized or service error
                         # This is probably caused by an expired token so the next retry will get a new one automatically
                         _LOGGER.debug("API got error code=%d key=%s - %d retries left", response.status, result.error_key, retry)
@@ -73,7 +100,7 @@ class HomeConnectApi():
                             _LOGGER.debug("API call failed with code=%d error=%s", response.status, result.error_key)
                         return result
             except Exception as ex:
-                _LOGGER.debug("API call to HomeConnect service failed", exc_info=ex)
+                _LOGGER.debug("HTTP call failed %s %s", method, endpoint, exc_info=ex)
                 if not retry:
                     raise HomeConnectError("API call to HomeConnect service failed", code=901, inner_exception=ex) from ex
             finally:
@@ -88,19 +115,19 @@ class HomeConnectApi():
 
     async def async_get(self, endpoint) -> ApiResponse:
         """ Implements a HTTP GET request """
-        return await self._async_request('get', endpoint)
+        return await self._async_request('GET', endpoint)
 
     async def async_put(self, endpoint:str, data:str) -> ApiResponse:
         """ Implements a HTTP PUT request """
-        return await self._async_request('put', endpoint, data=data)
+        return await self._async_request('PUT', endpoint, data=data)
 
     async def async_delete(self, endpoint:str) -> ApiResponse:
         """ Implements a HTTP DELETE request """
-        return await self._async_request('delete', endpoint)
+        return await self._async_request('DELETE', endpoint)
 
     async def async_get_event_stream(self, endpoint):
         """ Returns a Server Sent Events (SSE) stream to be consumed by the caller """
-        return await self._auth.stream(endpoint)
+        return await self._auth.stream(endpoint, self._lang)
 
 
     async def async_stream(self, endpoint:str, event_handler:Callable[[str], None]):
@@ -109,7 +136,7 @@ class HomeConnectApi():
         event_source = None
         while True:
             try:
-                event_source = await self._auth.stream(endpoint)
+                event_source = await self._auth.stream(endpoint, self._lang)
                 await event_source.connect()
                 async for event in event_source:
                     backoff = 1
