@@ -7,11 +7,11 @@ from dataclasses import dataclass, field
 import re
 from typing import Optional
 from dataclasses_json import dataclass_json, Undefined, config
-
+from home_connect_async.api import HomeConnectApi
 import home_connect_async.homeconnect as homeconnect
 import home_connect_async.callback_registery as callback_registery
 from .const import Events
-from .common import HomeConnectError
+from .common import HomeConnectError, Synchronization
 
 
 _LOGGER = logging.getLogger(__name__)
@@ -21,9 +21,10 @@ _LOGGER = logging.getLogger(__name__)
 class Status():
     """ Class to represent a Home Connect Status """
     key:str
-    name:Optional[str] = None
     value:Optional[any] = None
+    name:Optional[str] = None
     displayvalue:Optional[str] = None
+    unit:Optional[str] = None
 
     @classmethod
     def create(cls, data:dict):
@@ -32,7 +33,8 @@ class Status():
             key = data['key'],
             name = data.get('name'),
             value = data.get('value'),
-            displayvalue= data.get('displayvalue')
+            displayvalue= data.get('displayvalue'),
+            unit = data.get('unit')
         )
         return status
 
@@ -58,15 +60,18 @@ class Command():
 class Option():
     """ Class to represent a Home Connect Option """
     key:str
+    value:Optional[any] = None
     type:Optional[str] = None
     name:Optional[str] = None
     unit:Optional[str] = None
-    value:Optional[any] = None
     displayvalue:Optional[str] = None
     min:Optional[int] = None
     max:Optional[int] = None
     stepsize:Optional[int] = None
     allowedvalues:Optional[list[str]] = None
+    execution:Optional[str] = None
+    liveupdate:Optional[bool] = None
+    access:Optional[str] = None
 
     @classmethod
     def create(cls, data:dict):
@@ -85,6 +90,9 @@ class Option():
             option.max = constraints.get('max')
             option.stepsize = constraints.get('stepsize')
             option.allowedvalues = constraints.get('allowedvalues')
+            option.execution = constraints.get('execution')
+            option.liveupdate = constraints.get('liveupdate')
+            option.access = constraints.get('access')
         return option
 
     def get_option_to_apply(self, value, exception_on_error=False):
@@ -163,13 +171,75 @@ class Appliance():
     status:dict[str, Status] = None
     settings:dict[str, Option] = None
     commands:dict[str, Command] = None
+    startonly_options:dict[str, Option] = None
+    startonly_program:Optional[Program] = None
 
     # Internal fields
     _homeconnect:Optional[homeconnect.HomeConnect] = field(default_factory=lambda: None, metadata=config(encoder=lambda val: None, decoder=lambda val: None, exclude=lambda val: True))
-    #_api:Optional[HomeConnectApi] = field(default=None, metadata=config(encoder=lambda val: None, exclude=lambda val: True))
+    _api:Optional[HomeConnectApi] = field(default=None, metadata=config(encoder=lambda val: None, exclude=lambda val: True))
     _callbacks:Optional[callback_registery.CallbackRegistry] = field(default_factory=lambda: None, metadata=config(encoder=lambda val: None, exclude=lambda val: True))
 
+
+    #region - Helper functions
+    def get_applied_program(self) -> Program|None:
+        """ gets the currently applied program which is the active or startonly or selected program """
+        if self.active_program:
+            return self.active_program
+        elif self.startonly_program:
+            return self.startonly_program
+        elif self.selected_program:
+            return self.selected_program
+        else:
+            return None
+
+    def get_applied_program_option(self, option_key:str) -> Option|None:
+        prog = self.get_applied_program()
+        if prog and prog.options and option_key in prog.options:
+            return prog.options[option_key]
+        return None
+
+    def get_applied_program_available_options(self) -> dict[Option]|None:
+        """ gets the available options for the applied program """
+        prog = self.get_applied_program()
+        if prog and self.available_programs and prog.key in self.available_programs:
+            return self.available_programs[prog.key].options
+        else:
+            return None
+
+    def get_applied_program_available_option(self, option_key:str) -> Option|None:
+        """ gets a specific available option for the applied program """
+        opts = self.get_applied_program_available_options()
+        if opts and option_key in opts:
+            return opts[option_key]
+        else:
+            return None
+
+    def is_available_program(self, program_key:str) -> bool:
+        """ Test if the specified program is currently available """
+        return self.available_programs and program_key in self.available_programs
+
+    def is_available_option(self, option_key:str) -> bool:
+        """ Test if the specified option key is currently available for the applied program """
+        opt = self.get_applied_program_available_option(option_key)
+        return opt is not None
+
+    #endregion
+
     #region - Manage Programs
+    def set_startonly_option(self, option_key:str, value) -> None:
+        """ Set an option that will be used when starting the program """
+        if not self.startonly_options:
+            self.startonly_options = {}
+        if option_key not in self.startonly_options:
+            self.startonly_options[option_key] = Option(option_key, value=value)
+        else:
+            self.startonly_options[option_key].value = value
+
+    def clear_startonly_option(self, option_key:str) -> None:
+        """ Clear a previously set startonly option """
+        if self.startonly_options and option_key in self.startonly_options:
+            del self.startonly_options[option_key]
+
     async def async_get_active_program(self):
         """ Get the active program """
         prog = await self._async_fetch_programs('active')
@@ -182,24 +252,55 @@ class Appliance():
         self.selected_program = prog
         return prog
 
-    async def async_select_program(self, key:str=None, options:Sequence[dict]=None, program:Program=None) -> bool:
+    async def async_select_program(self, program_key:str=None, options:Sequence[dict]=None, validate:bool=True):
         """ Set the selected program
 
         Parameters:
         key: The key of the program to select
         options: Additional program options to set
-        program: A Program object that represents the selected program. If used then "key" is ignored.
         """
-        if program is not None:
-            key = program.key
 
-        if key is None:
-            _LOGGER.error('Either "program" or "key" must be specified')
-            return False
+        if  self.available_programs and program_key in self.available_programs:
+            program = self.available_programs[program_key]
+        elif validate:
+            _LOGGER.error("The selected program key is not available")
+            raise HomeConnectError("The selected program key is not available")
+        else:
+            program = None
 
-        return await self._async_set_program(key, options, 'selected')
+        previous_program = self.startonly_program if self.startonly_program else self.selected_program
+        if program:
+            if program.execution == 'startonly' and not self.active_program:
+                self.startonly_program = program
+                _LOGGER.debug("Setting startonly_program=%s", program.key)
+                if not previous_program or previous_program.key != program_key:
+                    await self._callbacks.async_broadcast_event(self, Events.PROGRAM_SELECTED, program_key)
+                return
+            else:
+                self.startonly_program = None
 
-    async def async_start_program(self, program_key:str=None, options:Sequence[dict]=None, program:Program=None) -> bool:
+        async with Synchronization.selected_program_lock:
+            if self.active_program:
+                res = await self._async_set_program(program_key, options, 'active')
+            else:
+                res = await self._async_set_program(program_key, options, 'selected')
+            if res and (not previous_program or previous_program.key != program_key):
+                # There is a race condition between this and the selected program event
+                # so check if it was alreayd update so we don't call twice
+                # Note that this can't be dropped because the new options notification may arrive before the
+                # program selected event and then the option values will not match the values that were there for the
+                # previous program
+                if self.active_program:
+                    self.active_program = await self._async_fetch_programs('active')
+                else:
+                    self.selected_program = await self._async_fetch_programs('selected')
+                #TODO: Consider if the above updates can be removed or if adding available_programs is required
+                self.available_programs = await self._async_fetch_programs('available')
+                await self._callbacks.async_broadcast_event(self, Events.PROGRAM_SELECTED, program_key)
+                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+
+
+    async def async_start_program(self, program_key:str=None, options:Sequence[dict]=None, validate:bool=True) -> bool:
         """ Started the specified program
 
         Parameters:
@@ -207,25 +308,29 @@ class Appliance():
         options: Additional program options to set
         program: A Program object that represents the selected program. If used then "key" is ignored.
         """
-        if program is not None:
-            program_key = program.key
 
         if not program_key:
-            if self.selected_program:
+            if self.startonly_program:
+                program_key = self.startonly_program.key
+            elif self.selected_program:
                 program_key = self.selected_program.key
             else:
-                _LOGGER.error('Either "program" or "key" must be specified')
-                return False
+                _LOGGER.error('"program_key" was not specified and there is no selected program to start')
+                raise HomeConnectError('Either "program" or "key" must be specified')
 
-        if not self.available_programs or program_key not in self.available_programs:
+        if validate and (not self.available_programs or program_key not in self.available_programs):
             _LOGGER.warning("The selected program in not one of the available programs (not supported by the API)")
             raise HomeConnectError("The specified program in not one of the available programs (not supported by the API)")
-            return False
 
-        if options is None and self.selected_program and self.available_programs:
+        if options is None:
             options = []
-            for opt in self.selected_program.options.values():
-                if opt.key in self.available_programs[program_key].options:
+            if self.selected_program and self.available_programs and not self.startonly_program:
+                for opt in self.selected_program.options.values():
+                    if opt.key in self.available_programs[program_key].options and (not self.startonly_options or opt.key not in self.startonly_options):
+                        option = { "key": opt.key, "value": opt.value}
+                        options.append(option)
+            if self.startonly_options:
+                for opt in self.startonly_options.values():
                     option = { "key": opt.key, "value": opt.value}
                     options.append(option)
 
@@ -237,8 +342,7 @@ class Appliance():
             await self.async_get_active_program()
         if self.active_program:
             endpoint = f'{self._base_endpoint}/programs/active'
-            _LOGGER.debug("Calling %s with delete verb", endpoint)
-            response = await self._homeconnect._api.async_delete(endpoint)
+            response = await self._api.async_delete(endpoint)
             if response.status == 204:
                 return True
             elif response.error_description:
@@ -268,6 +372,19 @@ class Appliance():
 
     async def async_set_option(self, option_key, value) -> bool:
         """ Set a value for a specific program option """
+        opt = self.get_applied_program_available_option(option_key)
+        if not opt:
+            _LOGGER.debug("Attempting to set unavailable option: %s", option_key)
+            _LOGGER.debug(self.available_programs)
+            raise ValueError("The option isn't currently available")
+
+        if opt.execution == "startonly":
+            if value:
+                self.set_startonly_option(option_key, value)
+            else:
+                self.clear_startonly_option(option_key)
+            return True
+
         return await self._async_set_service_value("options", option_key, value)
 
     async def async_apply_setting(self, setting_key, value) -> bool:
@@ -279,9 +396,14 @@ class Appliance():
         if service_type in ['settings', 'commands']:
             endpoint = f'{self._base_endpoint}/{service_type}/{key}'
         elif service_type == 'options':
-            endpoint = f'{self._base_endpoint}/programs/selected/options/{key}'
+            if self.active_program:
+                endpoint = f'{self._base_endpoint}/programs/active/options/{key}'
+            elif self.selected_program:
+                endpoint = f'{self._base_endpoint}/programs/selected/options/{key}'
+            else:
+                raise ValueError("No active/selected program to apply the options to")
         else:
-            raise ValueError(f"Unsupported service_type value: {service_type}")
+            raise ValueError(f"Unsupported service_type value: '{service_type}'")
 
         command = {
             "data": {
@@ -290,8 +412,7 @@ class Appliance():
             }
         }
         jscmd = json.dumps(command, indent=2)
-        _LOGGER.debug("Calling %s with:\n%s", endpoint, jscmd)
-        response = await self._homeconnect._api.async_put(endpoint, jscmd)
+        response = await self._api.async_put(endpoint, jscmd)
         if response.status == 204:
             return True
         elif response.error_description:
@@ -316,8 +437,7 @@ class Appliance():
                 command['data']['options'] = options
 
             jscmd = json.dumps(command, indent=2)
-            _LOGGER.debug("Calling %s with:\n%s", endpoint, jscmd)
-            response = await self._homeconnect._api.async_put(endpoint, jscmd)
+            response = await self._api.async_put(endpoint, jscmd)
             if response.status == 204:
                 return True
             elif response.error_key == "SDK.Error.UnsupportedOption":
@@ -346,67 +466,113 @@ class Appliance():
 
     #region - Handle Updates, Events and Callbacks
 
-    async def async_update_data(self, key:str, value) -> None:
+
+    async def async_update_data(self, data:dict) -> None:
         """ Update the appliance data model from a change event notification """
+        key:str = data["key"]
+        value = data["value"]
 
-        if key == 'BSH.Common.Root.SelectedProgram':
-            self.selected_program = await self._async_fetch_programs('selected')
+        if not self.connected:
+            # an event was received for a disconnected appliance, which means we didn"t get the CONNECTED event, so reload the appliace data
+            await self.async_fetch_data()
+            await self._callbacks.async_broadcast_event(self, Events.PAIRED)
             await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
-        elif key == 'BSH.Common.Root.ActiveProgram' and value:
-            self.active_program = await self._async_fetch_programs('active')
-            self.commands = await self._async_fetch_commands()
-            await self._callbacks.async_broadcast_event(self, Events.PROGRAM_STARTED)
-            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
-        elif ( (key == 'BSH.Common.Root.ActiveProgram' and not value) or key == 'BSH.Common.Event.ProgramFinished')  and self.active_program:
-            self.active_program = None
-            self.commands = await self._async_fetch_commands()
-            await self._callbacks.async_broadcast_event(self, Events.PROGRAM_FINISHED)
-            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
-        elif key in ['BSH.Common.Option.ProgramProgress', 'BSH.Common.Option.RemainingProgramTime']:
-            if self.active_program:
-                # no need to reload everything for progress events
-                self.active_program.options[key].value = value
-            else:
-                # apparently this can be a thing (getting progress notifications without getting the ActiveProgram event first)
-                self.active_program = await self._async_fetch_programs('active')
-                self.commands = await self._async_fetch_commands()
-                await self._callbacks.async_broadcast_event(self, Events.PROGRAM_STARTED)
-                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
-        elif key == 'BSH.Common.Status.OperationState':
-            is_new_state = self.status.get('BSH.Common.Status.OperationState') != value
-
-            if value == 'BSH.Common.EnumType.OperationState.Ready' \
-                and (not self.available_programs or len(self.available_programs) < 2):
-                # Workaround for the fact the API doesn't provide all the data (such as available programs)
-                # when a program is active, so if for some reason we were loaded with missing data reload it
-                old_available_programs_count = len(self.available_programs) if self.available_programs else 0
-                try:
-                    await self.async_fetch_data(include_static_data=True)
-                except HomeConnectError:
-                    pass
-                if old_available_programs_count != len(self.available_programs):
-                    await self._callbacks.async_broadcast_event(self, Events.PAIRED)
-                    await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
-
-            elif is_new_state:
-                await self.async_fetch_data(include_static_data=False)
-                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
         else:
             # update options, statuses and settings in the data model
-            # In all the below cases we have to call the API because the friendly names are not included in the event data
-            if self.selected_program and key in self.selected_program.options:
-                # self.selected_program.options[key].value = value
-                self.selected_program = await self._async_fetch_programs('selected')
-            if self.active_program and key in self.active_program.options:
-                #self.active_program.options[key].value = value
-                self.active_program = await self._async_fetch_programs('active')
-            if key in self.status:
-                #self.status[key].value = value
-                self.status = await self._async_fetch_status()
-            if key in self.settings:
-                #self.settings[key].value = value
-                self.settings = await self._async_fetch_settings()
+            if self.selected_program and self.selected_program.options and key in self.selected_program.options:
+                self.selected_program.options[key].value = value
+                self.selected_program.options[key].name = data.get("name")
+                self.selected_program.options[key].displayvalue = data.get("displayvalue")
+            if self.active_program and self.active_program.options and key in self.active_program.options:
+                self.active_program.options[key].value = value
+                self.active_program.options[key].name = data.get("name")
+                self.active_program.options[key].displayvalue = data.get("displayvalue")
 
+            if key in self.status:
+                self.status[key].value = value
+                self.status[key].name = data.get("name")
+                self.status[key].displayvalue = data.get("displayvalue")
+            elif "uri" in data and "/status/" in data["uri"]:
+                self.status = await self._async_fetch_status()
+                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+
+            if key in self.settings:
+                self.settings[key].value = value
+                self.settings[key].name = data.get("name")
+                self.settings[key].displayvalue = data.get("displayvalue")
+            elif "uri" in data and "/settings/" in data["uri"]:
+                self.settings = await self._async_fetch_settings()
+                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+
+        # Fetch data from the API on major events
+        if key == "BSH.Common.Root.SelectedProgram" and (not self.selected_program or self.selected_program.key != value):
+            # handle selected program
+            async with Synchronization.selected_program_lock:
+                # Have to check again after aquiring the lock
+                if key == "BSH.Common.Root.SelectedProgram" and (not self.selected_program or self.selected_program.key != value):
+                    if value:
+                        self.selected_program = await self._async_fetch_programs("selected")
+                        selected_key = self.selected_program.key
+                        if not self.available_programs:
+                            self.available_programs = await self._async_fetch_programs("available")
+                        elif selected_key in self.available_programs and self.available_programs[selected_key].options is None:
+                            self.available_programs[selected_key].options = await self._async_fetch_available_options(selected_key)
+                        else:
+                            _LOGGER.debug("Skipping fetch_available_options() for selected program")
+                        await self._callbacks.async_broadcast_event(self, Events.PROGRAM_SELECTED, value)
+                    else:
+                        self.selected_program = None
+                        #self.available_programs = await self._async_fetch_programs("available")
+                    await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+        elif ( (key == "BSH.Common.Root.ActiveProgram" and value) or
+                # apparently it is possible to get progress notifications without getting the ActiveProgram event first so we handle that
+                (key in ["BSH.Common.Option.ProgramProgress", "BSH.Common.Option.RemainingProgramTime"]) or
+                # it is also possible to get operation state Run without getting the ActiveProgram event
+                (key == "BSH.Common.Status.OperationState" and value=="BSH.Common.EnumType.OperationState.Run")
+            ) and \
+            (not self.active_program or (key == "BSH.Common.Root.ActiveProgram" and self.active_program.key != value) ) :
+            # handle program start
+            self.active_program = await self._async_fetch_programs("active")
+            self.available_programs = await self._async_fetch_programs("available")
+            self.commands = await self._async_fetch_commands()
+            await self._callbacks.async_broadcast_event(self, Events.PROGRAM_STARTED, value)
+            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+        elif ( (key == "BSH.Common.Root.ActiveProgram" and not value) or
+               (key == "BSH.Common.Status.OperationState" and value in ["BSH.Common.EnumType.OperationState.Ready", "BSH.Common.EnumType.OperationState.Finished"]) or
+               (key == "BSH.Common.Event.ProgramFinished")
+            ) and self.active_program:
+            # handle program end
+            prev_prog = self.active_program.key if self.active_program else None
+            self.active_program = None
+            self.commands = await self._async_fetch_commands()
+            self.available_programs = await self._async_fetch_programs("available")
+            await self._callbacks.async_broadcast_event(self, Events.PROGRAM_FINISHED, prev_prog)
+            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+        elif key == "BSH.Common.Status.OperationState" and \
+             value!="BSH.Common.EnumType.OperationState.Run" and \
+             self.status.get("BSH.Common.Status.OperationState") != value:  # ignore repeat notifiations of the same state
+            #await self.async_fetch_data(include_static_data=False)
+            self.active_program = await self._async_fetch_programs("active")
+            self.selected_program = await self._async_fetch_programs("selected")
+            self.available_programs = await self._async_fetch_programs("available")
+            self.commands = await self._async_fetch_commands()
+            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+        elif key =="BSH.Common.Status.RemoteControlStartAllowed":
+            self.available_programs = await self._async_fetch_programs("available")
+            await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+        elif ( not self.available_programs or len(self.available_programs) < 2) and \
+             ( key in ["BSH.Common.Status.OperationState", "BSH.Common.Status.RemoteControlActive"] ) and \
+             ( "BSH.Common.Status.OperationState" not in self.status or self.status["BSH.Common.Status.OperationState"].value == "BSH.Common.EnumType.OperationState.Ready" ) and \
+             ( "BSH.Common.Status.RemoteControlActive" not in self.status or self.status["BSH.Common.Status.RemoteControlActive"].value):
+            # Handle cases were the appliance data was loaded without getting all the programs (for example when HA is restarted while a program is active)
+            # If the state is Ready and remote control is possible and we didn"t load the available programs before then load them now
+            available_programs = await self._async_fetch_programs("available")
+            if available_programs:
+                self.available_programs = available_programs
+                await self._callbacks.async_broadcast_event(self, Events.PAIRED)
+                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
+
+        # broadcast the specific event that was received
         await self._callbacks.async_broadcast_event(self, key, value)
 
     def register_callback(self, callback:Callable[[Appliance, str, any], None], keys:str|Sequence[str] ) -> None:
@@ -450,6 +616,7 @@ class Appliance():
         )
         appliance._homeconnect = hc
         appliance._callbacks = hc._callbacks
+        appliance._api = hc._api
 
         await appliance.async_fetch_data()
 
@@ -468,23 +635,34 @@ class Appliance():
             await asyncio.sleep(delay)
         try:
             _LOGGER.debug("Starting to load appliance data for %s (%s)", self.name, self.haId)
-            if include_static_data:
-                available_programs = await self._async_fetch_programs('available')
-                if available_programs and (not self.available_programs or len(self.available_programs)<2):
-                    # Only update the available programs if we got new data
-                    self.available_programs = available_programs
 
             self.selected_program = await self._async_fetch_programs('selected')
             self.active_program = await self._async_fetch_programs('active')
             self.settings = await self._async_fetch_settings()
             self.status = await self._async_fetch_status()
             self.commands = await self._async_fetch_commands()
+            self.available_programs = await self._async_fetch_programs('available')
+            # if include_static_data or not self.available_programs:
+            #     if  (
+            #             'BSH.Common.Status.OperationState' not in self.status
+            #             or self.status['BSH.Common.Status.OperationState'].value == 'BSH.Common.EnumType.OperationState.Ready'
+            #         ) and (
+            #             'BSH.Common.Status.RemoteControlActive' not in self.status
+            #             or self.status['BSH.Common.Status.RemoteControlActive'].value):
+            #         # Only load the available programs if the state allows them to be loaded
+            #         available_programs = await self._async_fetch_programs('available')
+            #         if available_programs and (not self.available_programs or len(self.available_programs)<2):
+            #             # Only update the available programs if we got new data
+            #             self.available_programs = available_programs
+            #     else:
+            #         self.available_programs = None
+            #         _LOGGER.debug("Not loading available programs becuase BSH.Common.Status.OperationState=%s  and BSH.Common.Status.RemoteControlActive=%s",
+            #             self.status['BSH.Common.Status.OperationState'].value if 'BSH.Common.Status.OperationState' in self.status else None,
+            #             str(self.status['BSH.Common.Status.RemoteControlActive'].value) if 'BSH.Common.Status.RemoteControlActive' in self.status else None )
 
             _LOGGER.debug("Finished loading appliance data for %s (%s)", self.name, self.haId)
             if not self.connected:
                 await self.async_set_connection_state(True)
-            else:
-                await self._callbacks.async_broadcast_event(self, Events.DATA_CHANGED)
         except HomeConnectError as ex:
             if ex.error_key:
                 delay = delay + 60 if delay<300 else 300
@@ -495,19 +673,20 @@ class Appliance():
             _LOGGER.debug("Unexpected exception in Appliance.async_fetch_data", exc_info=ex)
             raise HomeConnectError("Unexpected exception in Appliance.async_fetch_data", inner_exception=ex)
 
+
     async def _async_fetch_programs(self, program_type:str):
         """ Main function to fetch the different kinds of programs with their options from the cloud service """
         endpoint = f'{self._base_endpoint}/programs/{program_type}'
-        response = await self._homeconnect._api.async_get(endpoint)
+        response = await self._api.async_get(endpoint)
         if response.error_key:
-            _LOGGER.debug("Failed to load Programs: %s with error code=%d key=%s", program_type, response.status, response.error_key)
+            _LOGGER.debug("Failed to load %s programs with error code=%d key=%s", program_type, response.status, response.error_key)
             return None
         elif not response.data:
             _LOGGER.debug("Didn't get any data for Programs: %s", program_type)
             raise HomeConnectError(msg=f"Failed to get a valid response from the Home Connect service ({response.status})", response=response)
         data = response.data
 
-        #programs = ProgramsDict()
+        current_program_key = self.active_program.key if self.active_program else self.selected_program.key if self.selected_program else None
         programs = {}
         if 'programs' not in data:
             # When fetching selected and active programs the parent program node doesn't exist so we force it
@@ -518,8 +697,10 @@ class Appliance():
             if 'options' in p:
                 options = self.optionlist_to_dict(p['options'])
                 _LOGGER.debug("Loaded %d Options for %s/%s", len(options), program_type, prog.key)
+            elif program_type=='available' and (prog.key == current_program_key or prog.execution == 'startonly'):
+                options = await self._async_fetch_available_options(prog.key)
             else:
-                options = await self._async_fetch_options(program_type, p['key'])
+                options = None
             prog.options = options
 
             programs[p['key']] = prog
@@ -532,33 +713,27 @@ class Appliance():
             _LOGGER.debug("Loaded %d available Programs", len(programs))
             return programs
 
-    async def _async_fetch_options(self, program_type:str, program_key:str=None):
+    async def _async_fetch_available_options(self, program_key:str=None):
         """ Fetch detailed options of a program """
-
-        # TODO: The program_type is not really used so it may make sense to clean this code up
-        if program_type=='available':
-            endpoint = f"{self._base_endpoint}/programs/available/{program_key}"
-        else:
-            endpoint = f"{self._base_endpoint}/programs/{program_type}/options"
-
-        response = await self._homeconnect._api.async_get(endpoint)      # This is expected to always succeed if the previous call succeeds
+        endpoint = f"{self._base_endpoint}/programs/available/{program_key}"
+        response = await self._api.async_get(endpoint)      # This is expected to always succeed if the previous call succeeds
         if response.error_key:
-            _LOGGER.debug("Failed to load Options of %s/%s with error code=%d key=%s", program_type, program_key, response.status, response.error_key)
+            _LOGGER.debug("Failed to load Options of available/%s with error code=%d key=%s", program_key, response.status, response.error_key)
             return None
         data = response.data
         if data is None or 'options' not in data:
-            _LOGGER.debug("Didn't get any data for Options of %s/%s", program_type, program_key)
+            _LOGGER.debug("Didn't get any data for Options of available/%s", program_key)
             return None
 
         options = self.optionlist_to_dict(data['options'])
-        _LOGGER.debug("Loaded %d Options for %s/%s", len(options), program_type, program_key)
+        _LOGGER.debug("Loaded %d Options for available/%s", len(options), program_key)
         return options
 
 
     async def _async_fetch_status(self):
         """ Fetch the appliance status values """
         endpoint = f'{self._base_endpoint}/status'
-        response = await self._homeconnect._api.async_get(endpoint)
+        response = await self._api.async_get(endpoint)
         if response.error_key:
             _LOGGER.debug("Failed to load Status with error code=%d key=%s", response.status, response.error_key)
             return {}
@@ -578,7 +753,7 @@ class Appliance():
     async def _async_fetch_settings(self):
         """ Fetch the appliance settings """
         endpoint = f'{self._base_endpoint}/settings'
-        response = await self._homeconnect._api.async_get(endpoint)
+        response = await self._api.async_get(endpoint)
         if response.error_key:
             _LOGGER.debug("Failed to load Settings with error code=%d key=%s", response.status, response.error_key)
             return {}
@@ -590,7 +765,7 @@ class Appliance():
         settings = {}
         for setting in data['settings']:
             endpoint = f'{self._base_endpoint}/settings/{setting["key"]}'
-            response = await self._homeconnect._api.async_get(endpoint)
+            response = await self._api.async_get(endpoint)
             if response.status != 200:
                 continue
             settings[setting['key']] = Option.create(response.data)
@@ -601,7 +776,7 @@ class Appliance():
     async def _async_fetch_commands(self):
         """ Fetch the appliance commands """
         endpoint = f'{self._base_endpoint}/commands'
-        response = await self._homeconnect._api.async_get(endpoint)
+        response = await self._api.async_get(endpoint)
         if response.error_key:
             _LOGGER.debug("Failed to load Settings with error code=%d key=%s", response.status, response.error_key)
             return {}
@@ -618,10 +793,10 @@ class Appliance():
         return commands
 
 
-    def optionlist_to_dict(self, l:Sequence[dict]) -> dict:
+    def optionlist_to_dict(self, options_list:Sequence[dict]) -> dict:
         """ Helper funtion to convert a list of options into a dictionary keyd by the option "key" """
         d = {}
-        for element in l:
+        for element in options_list:
             d[element['key']] = Option.create(element)
         return d
 
